<?php
	/*
	* ldap_abook class version 0.1
	*/

	define('SM_PATH', '../../');
	include_once(SM_PATH .'functions/i18n.php');
	
	class LdapInterface {

		var $connection;
		var $err;

		function connect($host, $port = null, $uid = null, $pwd = null, $version = null) {
			$this->err = new ERROR();
			$this->connection = @ldap_connect($host,$port);
			if (!$this->connection) {
				$n = ldap_errno($this->connection);
				$this->err->_code = $n;
				$this->err->_message = ldap_err2str($n);
				return $this->err;
			}
			else {
				if (isset($version)) {
					if (!@ldap_set_option($this->connection, LDAP_OPT_PROTOCOL_VERSION, $version)) {
						$n = ldap_errno($this->connection);
						$this->err->_code = $n;
						$this->err->_message = ldap_err2str($n);
						return $this->err;
					}
				}
				else {
					if (!@ldap_set_option($this->connection, LDAP_OPT_PROTOCOL_VERSION, 3))
						@ldap_set_option($this->connection, LDAP_OPT_PROTOCOL_VERSION, 2);
					$res = @ldap_bind($this->connection, $uid, $pwd);
					if (!$res) {
						$n = ldap_errno($this->connection);
						$this->err->_code = $n;
						$this->err->_message = ldap_err2str($n);
						return $this->err;
					}
				}
			}
			return 1;
		}

		function search($location,$query = '(objectClass=*)') {
			$res = @ldap_search($this->connection, $location, $query);
			if (!$res) {
				$n = ldap_errno($this->connection);
				$this->err->_code = $n;
				$this->err->_message = ldap_err2str($n);
				return $this->err;
			}		
			return $res;
		}

		function delete($dn) {
			$res = @ldap_delete($this->connection, $dn);
			if (!$res) {
				$n = ldap_errno($this->connection);
				$this->err->_code = $n;
				$this->err->_message = ldap_err2str($n);
				return $this->err;
			}
			return 1;
		}
		
		function modify($dn, $info) {
			$res = @ldap_modify($this->connection, $dn, $info);
			if (!$res) {
				$n = ldap_errno($this->connection);
				$this->err->_code = $n;
				$this->err->_message = ldap_err2str($n);
				return $this->err;
			}
			return 1;
		}
		
		function add($dn, $info) {
			$res = @ldap_add($this->connection, $dn, $info);
			if (!$res) {
				$n = ldap_errno($this->connection);
				$this->err->_code = $n;
				$this->err->_message = ldap_err2str($n);
				return $this->err;
			}
			return 1;
		}
		
		function _ldap_sort_function($a, $b) {
			if (!is_array($a) || !is_array($b) ||
				$a['sn'] == $b['sn']) return 0;
			return ($a['sn'] > $b['sn'])? 1 : -1;
		}
		
		function output_short($server, $location, $res, $sort=false) {
			$info_array = @ldap_get_entries($this->connection,$res);
			if (!$info_array) {
				$n = ldap_errno($this->connection);
				$this->err->_code = $n;
				$this->err->_message = ldap_err2str($n);
				return $this->err;
			}
			$count = ldap_count_entries($this->connection,$res);
			// Move the stupid count array
			for ($i = 0; $i < $info_array['count']; $i++)
				$tmp[] = $info_array[$i];
			$info_array = $tmp;
			if (isset($sort) && $sort) {
				usort($info_array, array($this, '_ldap_sort_function'));
				if ($sort == 'DESC')
					$info = array_reverse($info_array);
				else
					$info = $info_array;
			}
			else
				foreach ($info_array as $row) {
						$info[] = $row;
				}
			bindtextdomain('ldap_abook', SM_PATH . 'plugins/ldap_abook/locale');
   			textdomain('ldap_abook');
			print '<p style="font-size: 0.8em; text-align: center;">';
			print _('Number of entries found');
			print ': '.$count;
			print '</p><p style="font-size: 0.8em;"><a href="ldap_abook.php?new=';
			print urlencode($location);
			print '&amp;server='.$server.'">';
			print _('Add new Contact');
			print '</a></p><table bgcolor="lightgray" align="center" cellspacing="2" cellpadding="5">';
      		print '<tr><th style="color: black;background-color: #DAF9FF;">';
			print _('Name');
			print '</th><th style="color: black;background-color: #DAF9FF;">';
			print _('Email');
			print '</th><th style="color: black;background-color: #DAF9FF;">';
			print _('Home Phone');
			print '</th><th style="color: black;background-color: #DAF9FF;">';
			print _('Mobile Phone');
			print '</th><th style="color: black;background-color: #DAF9FF;">&nbsp;</th></tr>';
 			foreach ($info as $record) {
				print '<tr><td style="font-size: 0.8em;font-family: verdana times san-serif;color: navy;background-color: #FFF693;">';
				foreach ($record['cn'] as $cn)
					if (is_string($cn))
						print directory_string_convert($cn, $server, 'SM');
				print '</td>';
         		print '<td style="font-size: 0.8em;font-family: verdana times san-serif;color: navy;background-color: #FFF693;">';
				foreach ($record['mail'] as $mail) {
					if (is_string($mail)) {
						if ($compose_new_win == '1') {
			    			print '<a href=src/compose.php'.
			            	'?send_to=$mail target="compose_window" '.
				   			'onClick="comp_in_new()">'.$mail.'</a>';
						}
						else {
                    		displayInternalLink ('src/compose.php?send_to='.urlencode($mail),$mail, 'right');
						}
					}
				}
				print '</td>';
         		print '<td style="font-size: 0.8em;font-family: verdana times san-serif;color: navy;background-color: #FFF693;">';
				foreach ($record['homephone'] as $hp)
					if (is_string($hp))
						print $hp;
				print '</td>';
         		print '<td style="font-size: 0.8em;font-family: verdana times san-serif;color: navy;background-color: #FFF693;">';
				foreach ($record['mobile'] as $mp)
					if (is_string($mp))
						print $mp;
				print '</td>';
         		print '<td style="font-size: 0.8em;font-family: verdana times san-serif;color: navy;background-color: #FFF693;">';
				print '<a href="ldap_abook.php?delete='.
				urlencode($record['dn']).'&amp;server='.
				$server.'">'._('Del').'</a> | <a href="ldap_abook.php?modify='.
				urlencode($record['dn']).'&amp;server='.
				$server.'">'._('Mod').'</a>';
				print '</td></tr>';
			}
			print '</table><p style="font-size: 0.8em"><a href="ldap_abook.php?new='.
			urlencode($location).'&amp;server='.$server.
			'">'._('Add new Contact').'</a></p>';
			bindtextdomain('squirrelmail', SM_PATH . 'locale');
   			textdomain('squirrelmail');
			return 1;
		}

		function close() {
			@ldap_close($this->connection);
		}
		
		function is_error($val) {
			/***********************************************/
			/* is_object returns/uses class names in lower */
			/* case! bug? The same goes for is_subclass_of */
			/***********************************************/

			return is_object($val) &&
			((get_class($val) == 'error') ||
			is_subclass_of($val, 'error'));
		}
		
	}
			
	class ERROR {

		var $_code;
		var $_message;
		
		function getCode() {
			return $this->_code;
		}

		function getMessage() {
			return $this->_message;
		}

	}
	
?>
